@@ -82,6 +82,8 @@ class BaseModel(pl.LightningModule):
         self.val_dataloader_names = []
         self.val_dataloader_lengths = []
 
+        self.test_dataloader_names = []
+
         self.last_inputs = None
         self.last_predictions = None
 
@@ -136,6 +138,10 @@ class BaseModel(pl.LightningModule):
             help=('String specifying the dataset for validation. See the docs of '
                   'ptlflow.models.base_model.base_model.BaseModel.parse_dataset_selection for more details about this '
                   'string.'))
+        parser.add_argument(
+            '--test_dataset', type=str, nargs='+', default=None, choices=['kitti-2012', 'kitti-2015', 'sintel'],
+            help=('Specify the datasets for testing. This option should be used to generate predictions on the'
+                  'test set of datasets that have official benchmarks.'))
         parser.add_argument(
             '--pretrained_ckpt', type=str, default=None,
             help=('A string identifier of the pretrained checkpoint to load. The string will be first checked against the '
@@ -336,21 +342,20 @@ class BaseModel(pl.LightningModule):
             if self.args.max_epochs is None:
                 self.args.max_epochs = 10
                 logging.warning('--max_epochs is not set. It will be set to %d.', self.args.max_epochs)
-            self.args.max_steps = self.args.max_epochs * self.train_dataloader_length
 
-        divider = self.args.gpus
-        if isinstance(divider, list) or isinstance(divider, tuple):
-            divider = len(divider)
-        elif isinstance(divider, str):
-            divider = len(divider.split(','))
-        elif not isinstance(divider, int):
-            divider = 1
+            gpu_divider = self.args.gpus
+            if isinstance(gpu_divider, list) or isinstance(gpu_divider, tuple):
+                gpu_divider = len(gpu_divider)
+            elif isinstance(gpu_divider, str):
+                gpu_divider = len(gpu_divider.split(','))
+            elif not isinstance(gpu_divider, int):
+                gpu_divider = 1
 
-        total_steps = self.args.max_steps // divider
+            self.args.max_steps = self.args.max_epochs * (self.train_dataloader_length // gpu_divider)
 
         optimizer = optim.AdamW(self.parameters(), lr=self.args.lr, weight_decay=self.args.wdecay)
         lr_scheduler = optim.lr_scheduler.OneCycleLR(
-            optimizer, self.args.lr, total_steps=total_steps, pct_start=0.05, cycle_momentum=False, anneal_strategy='linear')
+            optimizer, self.args.lr, total_steps=self.args.max_steps, pct_start=0.05, cycle_momentum=False, anneal_strategy='linear')
         return {'optimizer': optimizer,
                 'lr_scheduler': {'scheduler': lr_scheduler, 'interval': 'step'}}
 
@@ -433,7 +438,8 @@ class BaseModel(pl.LightningModule):
             valid dataset strings.
         """
         if self.args.val_dataset is None:
-            self.args.val_dataset = 'sintel-final-trainval+sintel-clean-trainval+kitti-2012-trainval+kitti-2015-trainval'
+            self.args.val_dataset = (
+                'sintel-final-trainval-occ+sintel-clean-trainval-occ+kitti-2012-trainval+kitti-2015-trainval')
             logging.warning(
                 '--val_dataset is not set. It will be set as %s. If you want to skip validation, then set --val_dataset none.',
                 self.args.val_dataset)
@@ -454,6 +460,40 @@ class BaseModel(pl.LightningModule):
 
         return dataloaders
 
+    def test_dataloader(self) -> List[DataLoader]:
+        """Initialize and return the list of test dataloaders.
+
+        The list of datasets to load is provided by the --test_dataset argument.
+
+        Returns
+        -------
+        Optional[List[DataLoader]]
+            A list of dataloaders each for one dataset.
+
+        Raises
+        ------
+        ValueError
+            If --test_dataset is not provided.
+        """
+        if self.args.test_dataset is None:
+            raise ValueError('--test_dataset is not set. Please inform at least one dataset for the testing.')
+
+        dataset_ids = self.args.test_dataset
+        if 'sintel' in dataset_ids:
+            dataset_ids.remove('sintel')
+            dataset_ids.extend(['sintel-clean', 'sintel-final'])
+
+        dataloaders = []
+        for dataset_id in self.args.test_dataset:
+            dataset_id += '-test'
+            dataset_tokens = dataset_id.split('-')
+            dataset = getattr(self, f'_get_{dataset_tokens[0]}_dataset')(False, *dataset_tokens[1:])
+            dataloaders.append(DataLoader(dataset, 1, shuffle=False, num_workers=1, pin_memory=False, drop_last=False))
+
+            self.test_dataloader_names.append(dataset_id)
+
+        return dataloaders
+
     def parse_dataset_selection(
         self,
         dataset_selection: str,
@@ -627,11 +667,26 @@ class BaseModel(pl.LightningModule):
             transform = ft.ToTensor()
 
         split = 'trainval'
-        if len(args) > 0 and args[0] in ['train', 'val', 'trainval']:
-            split = args[0]
+        add_reverse = False
+        get_occlusion_mask = False
+        get_motion_boundary_mask = False
+        get_backward = False
+        for v in args:
+            if v in ['train', 'val', 'trainval']:
+                split = v
+            elif v == 'rev':
+                add_reverse = True
+            elif v == 'occ':
+                get_occlusion_mask = True
+            elif v == 'mb':
+                get_motion_boundary_mask = True
+            elif v == 'back':
+                get_backward = True
+
         dataset = FlyingChairs2Dataset(
-            self.args.flying_chairs2_root_dir, split=split, transform=transform, add_reverse=True, get_occlusion_mask=False,
-            get_motion_boundary_mask=False, get_backward=False)
+            self.args.flying_chairs2_root_dir, split=split, transform=transform, add_reverse=add_reverse,
+            get_occlusion_mask=get_occlusion_mask, get_motion_boundary_mask=get_motion_boundary_mask,
+            get_backward=get_backward)
         return dataset
 
     def _get_hd1k_dataset(
@@ -700,12 +755,11 @@ class BaseModel(pl.LightningModule):
 
         versions = ['2012', '2015']
         split = 'trainval'
-        if len(args) > 0:
-            for v in args:
-                if v in ['2012', '2015']:
-                    versions = [v]
-                elif v in ['train', 'val', 'trainval', 'test']:
-                    split = v
+        for v in args:
+            if v in ['2012', '2015']:
+                versions = [v]
+            elif v in ['train', 'val', 'trainval', 'test']:
+                split = v
 
         dataset = KittiDataset(
             self.args.kitti_2012_root_dir, self.args.kitti_2015_root_dir, versions=versions, split=split, transform=transform)
@@ -742,16 +796,18 @@ class BaseModel(pl.LightningModule):
 
         pass_names = ['clean', 'final']
         split = 'trainval'
-        if len(args) > 0:
-            for v in args:
-                if v in ['clean', 'final']:
-                    pass_names = [v]
-                elif v in ['train', 'val', 'trainval', 'test']:
-                    split = v
+        get_occlusion_mask = False
+        for v in args:
+            if v in ['clean', 'final']:
+                pass_names = [v]
+            elif v in ['train', 'val', 'trainval', 'test']:
+                split = v
+            elif v == 'occ':
+                get_occlusion_mask = True
 
         dataset = SintelDataset(
             self.args.mpi_sintel_root_dir, split=split, pass_names=pass_names, transform=transform,
-            get_occlusion_mask=False)
+            get_occlusion_mask=get_occlusion_mask)
         return dataset
 
     def _get_things_dataset(
@@ -786,25 +842,38 @@ class BaseModel(pl.LightningModule):
         pass_names = ['clean', 'final']
         split = 'trainval'
         is_subset = False
-        if len(args) > 0:
-            for v in args:
-                if v in ['clean', 'final']:
-                    pass_names = [v]
-                elif v in ['train', 'val', 'trainval']:
-                    split = v
-                elif v == 'subset':
-                    is_subset = True
+        add_reverse = False
+        get_occlusion_mask = False
+        get_motion_boundary_mask = False
+        get_backward = False
+        for v in args:
+            if v in ['clean', 'final']:
+                pass_names = [v]
+            elif v in ['train', 'val', 'trainval']:
+                split = v
+            elif v == 'subset':
+                is_subset = True
+            elif v == 'rev':
+                add_reverse = True
+            elif v == 'occ':
+                get_occlusion_mask = True
+            elif v == 'mb':
+                get_motion_boundary_mask = True
+            elif v == 'back':
+                get_backward = True
 
         if is_subset:
             dataset = FlyingThings3DSubsetDataset(
                 self.args.flying_things3d_subset_root_dir, split=split, pass_names=pass_names,
-                side_names=['left', 'right'], add_reverse=True, transform=transform,
-                get_occlusion_mask=False, get_motion_boundary_mask=False, get_backward=False)
+                side_names=['left', 'right'], add_reverse=add_reverse, transform=transform,
+                get_occlusion_mask=get_occlusion_mask, get_motion_boundary_mask=get_motion_boundary_mask,
+                get_backward=get_backward)
         else:
             dataset = FlyingThings3DDataset(
                 self.args.flying_things3d_root_dir, split=split, pass_names=pass_names,
-                side_names=['left', 'right'], add_reverse=True, transform=transform,
-                get_occlusion_mask=False, get_motion_boundary_mask=False, get_backward=False)
+                side_names=['left', 'right'], add_reverse=add_reverse, transform=transform,
+                get_occlusion_mask=get_occlusion_mask, get_motion_boundary_mask=get_motion_boundary_mask,
+                get_backward=get_backward)
         return dataset
 
     def _get_overfit_dataset(
